package de.mindcrimeilab.blog.purexsltrans;

import javax.xml.transform.ErrorListener;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;
import java.io.File;
import java.io.FileNotFoundException;
import java.net.URL;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Simple class used to execute the sample transformations with different XSLT processors based on the JAXP plugability
 * layer.
 * <p/>
 * Desired transformer factory is set out side using the system parameter {@code javax.xml.transform.TransformerFactory}.
 */
public class XslTransform {
    /**
     * logging instance
     */
    private final static Logger LOGGER = Logger.getLogger(XslTransform.class.getCanonicalName());

    /**
     * initialise logger quickly
     */
    static {
        // log anything
        LOGGER.setLevel(Level.ALL);
    }

    /**
     * Loads the XSL file from the classpath and starts the transformation of the sample xml file 'sample-order.xml'
     * printing the result to the standard output.
     *
     * @param args no arguments are processed by the implementation
     * @throws TransformerException  in case of any transformation error
     * @throws FileNotFoundException if there will be any I/O error accessing the XSL file
     */
    public static void main(String... args) throws TransformerException, FileNotFoundException {
        // prepare error listener
        ErrorListener el = new ErrorListenerImpl();
        LOGGER.info("Starting transformation...");

        // try loading stylesheet from classpath using current class loader
        ClassLoader ccl = Thread.currentThread().getContextClassLoader();
        URL url = ccl.getResource("order-transformation.xsl");

        LOGGER.info(String.format("Loading xsl %s", url));

        if (null == url) {
            LOGGER.severe("Could not load xsl transformation file.");
            throw new FileNotFoundException("Could not load xsl transformation file.");
        }

        // create transformer factory instance as specified by environment (this will switch between the different
        // library implementations
        TransformerFactory tFactory = TransformerFactory.newInstance();
        // attach error listener to get informed about possible malfunctions
        tFactory.setErrorListener(el);

        LOGGER.info(String.format("Using transformer factory: %s", tFactory.getClass().getCanonicalName()));

        Transformer transformer = tFactory.newTransformer(new StreamSource(url.getFile()));

        // prepare stylesheet parameters {@see order-transformation.xsl} for details
        transformer.setParameter("resPathPrefix", "../../../i18n"); // prefix to load i18n resources from
        transformer.setParameter("lang", "dee"); // wrong param for testing - there is no file 'resources_dee.xml'

        // attach error listener too
        transformer.setErrorListener(el);

        // execute transformation; write output to standard out.
        transformer.transform(new StreamSource(new File("sample-order.xml")), new StreamResult(System.out));


    }

    /**
     * Inner class providing a simple error logger using 'global' logger. For more information {@see ErrorListener}
     */
    private static class ErrorListenerImpl implements ErrorListener {

        @Override
        public void warning(TransformerException e) throws TransformerException {
            LOGGER.warning(e.getMessageAndLocation());
        }

        @Override
        public void error(TransformerException e) throws TransformerException {
            LOGGER.severe(e.getMessageAndLocation());
        }

        @Override
        public void fatalError(TransformerException e) throws TransformerException {
            LOGGER.severe(e.getMessageAndLocation());
        }
    }
}

